#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-03-14
#
# Monitor status of Check Point SecureXL
#
# 2020-06-08: changed snmp-scan function
#             renamed from checkpoint_securexl_status to checkpoint_securexl
# 2021-08-26: rewritten for CMK 2.0
#
# snmpwalk sample
#
# CHECKPOINT-MIB::fwSXLStatus.0 = INTEGER: enabled(1)
# CHECKPOINT-MIB::fwSXLConnsExisting.0 = Wrong Type (should be INTEGER): Gauge32: 51
# CHECKPOINT-MIB::fwSXLConnsAdded.0 = Wrong Type (should be INTEGER): Gauge32: 630
# CHECKPOINT-MIB::fwSXLConnsDeleted.0 = Wrong Type (should be INTEGER): Gauge32: 578
# 
# .1.3.6.1.4.1.2620.1.36.1.1.0 = INTEGER: 1
# .1.3.6.1.4.1.2620.1.36.1.2.0 = Gauge32: 40
# .1.3.6.1.4.1.2620.1.36.1.3.0 = Gauge32: 645
# .1.3.6.1.4.1.2620.1.36.1.4.0 = Gauge32: 604
#
# sample info
#
# [[u'1', u'48', u'7932', u'7986']]
#
import time
from dataclasses import dataclass
from typing import Optional

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    Service,
    check_levels,
    SNMPTree,
    all_of,
    startswith,
    any_of,
    equals,
    get_rate,
    GetRateError,
    get_value_store,
    Result,
    State,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    DiscoveryResult,
    CheckResult,
    StringTable,
)


@dataclass
class CheckpointSecurexl:
    fwSXLStatus: int
    fwSXLConnsExisting: int
    fwSXLConnsAdded: int
    fwSXLConnsDeleted: int


def parse_checkpoint_securexl(string_table: StringTable) -> Optional[CheckpointSecurexl]:
    try:
        wSXLStatus, fwSXLConnsExisting, fwSXLConnsAdded, fwSXLConnsDeleted = string_table[0]
    except (ValueError, IndexError):
        return

    return CheckpointSecurexl(
        fwSXLStatus=int(wSXLStatus),
        fwSXLConnsExisting=int(fwSXLConnsExisting),
        fwSXLConnsDeleted=int(fwSXLConnsDeleted),
        fwSXLConnsAdded=int(fwSXLConnsAdded),
    )


def discovery_checkpoint_securexl(section: CheckpointSecurexl) -> DiscoveryResult:
    yield Service()


def check_checkpoint_securexl(section: CheckpointSecurexl) -> CheckResult:
    if section.fwSXLStatus == 1:

        now_time = time.time()
        value_store = get_value_store()

        yield from check_levels(
            value=section.fwSXLConnsExisting,
            label='Connections active',
            metric_name='checkpoint_securexl_active',
            render_func=lambda v: f'{v:.0f}'
        )

        for label, metric, value in [
            ('added', 'added', section.fwSXLConnsAdded),
            ('deleted', 'deleted', section.fwSXLConnsDeleted),
        ]:
            try:
                value = get_rate(value_store, f'checkpoint_securexl_{metric}', now_time, value, raise_overflow=True)
            except GetRateError:
                value = 0
            yield from check_levels(
                value=value,
                label=label,
                metric_name=f'checkpoint_securexl_{metric}',
                render_func=lambda v: f'{v:0.2f}/s',
            )
    else:
        yield Result(state=State.WARN, notice='SXL not enabled')


register.snmp_section(
    name='checkpoint_securexl',
    parse_function=parse_checkpoint_securexl,
    fetch=SNMPTree(
        base='.1.3.6.1.4.1.2620.1.36.1',  #
        oids=[
            '1',  # fwSXLStatus
            '2',  # fwSXLConnsExisting
            '3',  # fwSXLConnsAdded
            '4',  # fwSXLConnsDeleted
        ]
    ),
    detect=any_of(
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.2620'),
        all_of(
            equals('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.8072.3.2.10'),
            equals('.1.3.6.1.4.1.2620.1.6.1.0', 'SVN Foundation'),
        )
    )
)

register.check_plugin(
    name='checkpoint_securexl',
    service_name='SecureXL',
    discovery_function=discovery_checkpoint_securexl,
    check_function=check_checkpoint_securexl,
)
